# -*- coding: utf-8 -*-

# PLEASE DO NOT EDIT THIS FILE, IT IS GENERATED AND WILL BE OVERWRITTEN:
# https://github.com/ccxt/ccxt/blob/master/CONTRIBUTING.md#how-to-contribute-code

from ccxt.async_support.base.exchange import Exchange
from ccxt.abstract.bitpin import ImplicitAPI
from ccxt.base.types import Int, Market, OrderBook, Strings, Ticker, Tickers
from typing import List


class bitpin(Exchange, ImplicitAPI):

    def describe(self):
        return self.deep_extend(super(bitpin, self).describe(), {
            'id': 'bitpin',
            'name': 'bitpin',
            'country': ['IR'],
            'rateLimit': 1000,
            'version': '1',
            'certified': False,
            'pro': False,
            'has': {
                'CORS': None,
                'spot': True,
                'margin': False,
                'swap': False,
                'future': False,
                'option': False,
                'addMargin': False,
                'cancelAllOrders': False,
                'cancelOrder': False,
                'cancelOrders': False,
                'createDepositAddress': False,
                'createOrder': False,
                'createStopLimitOrder': False,
                'createStopMarketOrder': False,
                'createStopOrder': False,
                'editOrder': False,
                'fetchBalance': False,
                'fetchBorrowInterest': False,
                'fetchBorrowRateHistories': False,
                'fetchBorrowRateHistory': False,
                'fetchClosedOrders': False,
                'fetchCrossBorrowRate': False,
                'fetchCrossBorrowRates': False,
                'fetchCurrencies': False,
                'fetchDepositAddress': False,
                'fetchDeposits': False,
                'fetchFundingHistory': False,
                'fetchFundingRate': False,
                'fetchFundingRateHistory': False,
                'fetchFundingRates': False,
                'fetchIndexOHLCV': False,
                'fetchIsolatedBorrowRate': False,
                'fetchIsolatedBorrowRates': False,
                'fetchL2OrderBook': False,
                'fetchL3OrderBook': False,
                'fetchLedger': False,
                'fetchLedgerEntry': False,
                'fetchLeverageTiers': False,
                'fetchMarkets': True,
                'fetchMarkOHLCV': False,
                'fetchMyTrades': False,
                'fetchOHLCV': True,
                'fetchOpenInterestHistory': False,
                'fetchOpenOrders': False,
                'fetchOrder': False,
                'fetchOrderBook': True,
                'fetchOrders': False,
                'fetchOrderTrades': 'emulated',
                'fetchPositions': False,
                'fetchPremiumIndexOHLCV': False,
                'fetchTicker': True,
                'fetchTickers': True,
                'fetchTime': False,
                'fetchTrades': False,
                'fetchTradingFee': False,
                'fetchTradingFees': False,
                'fetchWithdrawals': False,
                'setLeverage': False,
                'setMarginMode': False,
                'transfer': False,
                'withdraw': False,
            },
            'comment': 'This comment is optional',
            'urls': {
                'logo': 'https://cdn.arz.digital/cr-odin/img/exchanges/bitpin/64x64.png',
                'api': {
                    'public': 'https://api.bitpin.ir',
                    'OHLCV': 'https://api.bitpin.org',
                },
                'www': 'https://bitpin.ir',
                'doc': [
                    'https://docs.bitpin.ir',
                ],
            },
            'timeframes': {
                '1m': '1',
                '5m': '5',
                '15m': '15',
                '30m': '30',
                '1h': '60',
                '3h': '180',
                '4h': '240',
                '12h': '720',
                '1d': '1D',
                '1w': '1W',
            },
            'api': {
                'public': {
                    'get': {
                        'v1/mkt/markets/': 1,
                        'v2/mth/actives/': 1,
                        'v1/mkt/tv/get_bars/': 1,
                    },
                },
            },
            'fees': {
                'trading': {
                    'tierBased': False,
                    'percentage': True,
                    'maker': self.parse_number('0.001'),
                    'taker': self.parse_number('0.001'),
                },
            },
        })

    async def fetch_markets(self, symbols: Strings = None, params={}) -> List[Market]:
        """
        retrieves data on all markets for bitpin
        :see: https://api-docs.bitpin.ir/#be8d9c51a2
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict[]: an array of objects representing market data
        """
        response = await self.publicGetV1MktMarkets(params)
        markets = self.safe_dict(response, 'results')
        result = []
        for i in range(0, len(markets)):
            market = await self.parse_market(markets[i])
            result.append(market)
        return result

    def parse_market(self, market) -> Market:
        id = self.safe_string(market, 'id')
        baseCurrency = self.safe_dict(market, 'currency1')
        quoteCurrency = self.safe_dict(market, 'currency2')
        baseId = self.safe_string(baseCurrency, 'code')
        quoteId = self.safe_string(quoteCurrency, 'code')
        base = self.safe_currency_code(baseId)
        quote = self.safe_currency_code(quoteId)
        baseId = baseId.lower()
        quoteId = quoteId.lower()
        return {
            'id': id,
            'symbol': base + '/' + quote,
            'base': base,
            'quote': quote,
            'settle': None,
            'baseId': baseId,
            'quoteId': quoteId,
            'settleId': None,
            'type': 'spot',
            'spot': True,
            'margin': False,
            'swap': False,
            'future': False,
            'option': False,
            'active': True,
            'contract': False,
            'linear': None,
            'inverse': None,
            'contractSize': None,
            'expiry': None,
            'expiryDatetime': None,
            'strike': None,
            'optionType': None,
            'precision': {
                'amount': None,
                'price': None,
            },
            'limits': {
                'leverage': {
                    'min': None,
                    'max': None,
                },
                'amount': {
                    'min': None,
                    'max': None,
                },
                'price': {
                    'min': None,
                    'max': None,
                },
                'cost': {
                    'min': None,
                    'max': None,
                },
            },
            'created': None,
            'info': market,
        }

    async def fetch_tickers(self, symbols: Strings = None, params={}) -> Tickers:
        """
        fetches price tickers for multiple markets, statistical information calculated over the past 24 hours for each market
        :see: https://api-docs.bitpin.ir/#be8d9c51a2
        :param str[]|None symbols: unified symbols of the markets to fetch the ticker for, all market tickers are returned if not assigned
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a dictionary of `ticker structures <https://docs.ccxt.com/#/?id=ticker-structure>`
        """
        await self.load_markets()
        if symbols is not None:
            symbols = self.market_symbols(symbols)
        response = await self.publicGetV1MktMarkets(params)
        markets = self.safe_dict(response, 'results')
        result = {}
        for i in range(0, len(markets)):
            is_active = self.safe_bool(markets[i], 'tradable')
            if is_active is True:
                ticker = await self.parse_ticker(markets[i])
                symbol = ticker['symbol']
                result[symbol] = ticker
        return self.filter_by_array_tickers(result, 'symbol', symbols)

    async def fetch_ticker(self, symbol: str, params={}) -> Ticker:
        """
        fetches a price ticker, a statistical calculation with the information calculated over the past 24 hours for a specific market
        :see: https://api-docs.bitpin.ir/#be8d9c51a2
        :param str symbol: unified symbol of the market to fetch the ticker for
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a `ticker structure <https://docs.ccxt.com/#/?id=ticker-structure>`
        """
        ticker = await self.fetch_tickers([symbol])
        return ticker[symbol]

    def parse_ticker(self, ticker, market: Market = None) -> Ticker:
        # {
        #     'id': 1,
        #     'currency1': {
        #         'id': 1,
        #         'title': 'Bitcoin',
        #         'title_fa': 'بیت کوین',
        #         'code': 'BTC',
        #         'tradable': True,
        #         'for_test': False,
        #         'image': 'https://cdn.bitpin.ir/media/market/currency/1697370601.svg',
        #         'decimal': 2,
        #         'decimal_amount': 8,
        #         'decimal_irt': 0,
        #         'color': 'f7931a',
        #         'high_risk': False,
        #         'show_high_risk': False,
        #         'withdraw_commission': '0.003000000000000000',
        #         'tags': [
        #             {
        #                 'id': 44,
        #                 'name': 'لایه ۱',
        #                 'name_en': 'layer-1',
        #                 'has_chart': True,
        #             },
        #             {
        #                 'id': 52,
        #                 'name': 'اثبات کار',
        #                 'name_en': 'pow',
        #                 'has_chart': True,
        #             },
        #         ],
        #         'etf': False,
        #         'for_binvest': False,
        #         'for_loan': True,
        #         'for_stake': False,
        #         'recommend_for_deposit_weight': 1,
        #     },
        #     'currency2': {
        #         'id': 2,
        #         'title': 'Toman',
        #         'title_fa': 'تومان',
        #         'code': 'IRT',
        #         'tradable': True,
        #         'for_test': False,
        #         'image': 'https://cdn.bitpin.ir/media/market/currency/1684671406.svg',
        #         'decimal': 0,
        #         'decimal_amount': 0,
        #         'decimal_irt': 1,
        #         'color': '00fd22',
        #         'high_risk': False,
        #         'show_high_risk': False,
        #         'withdraw_commission': '0.000200000000000000',
        #         'tags': [],
        #         'etf': False,
        #         'for_binvest': False,
        #         'for_loan': False,
        #         'for_stake': False,
        #         'recommend_for_deposit_weight': 0,
        #     },
        #     'tradable': True,
        #     'for_test': False,
        #     'otc_sell_percent': '0.01000',
        #     'otc_buy_percent': '0.01000',
        #     'otc_max_buy_amount': '0.017000000000000000',
        #     'otc_max_sell_amount': '0.017000000000000000',
        #     'order_book_info': {
        #         'created_at': null,
        #         'price': '3894924262',
        #         'change': 0.0179,
        #         'min': '3777777800',
        #         'max': '3925000000',
        #         'time': '2024-05-19T13:45:00.000Z',
        #         'mean': '3833950912',
        #         'value': '6215833783',
        #         'amount': '1.62286922',
        #     },
        #     'internal_price_info': {
        #         'created_at': 1716126301.298626,
        #         'price': '3894924262',
        #         'change': 1.8,
        #         'min': '3777777800',
        #         'max': '3925000000',
        #         'time': null,
        #         'mean': null,
        #         'value': null,
        #         'amount': null,
        #     },
        #     'price_info': {
        #         'created_at': 1716126370.677,
        #         'price': '3906940950',
        #         'change': 2.04,
        #         'min': '3785113135',
        #         'max': '3921003333',
        #         'time': null,
        #         'mean': null,
        #         'value': null,
        #         'amount': null,
        #     },
        #     'price': '3906940950',
        #     'title': 'Bitcoin/Toman',
        #     'code': 'BTC_IRT',
        #     'title_fa': 'بیت کوین/تومان',
        #     'trading_view_source': 'BINANCE',
        #     'trading_view_symbol': 'BTCUSDT',
        #     'otc_market': False,
        #     'text': '',
        #     'volume_24h': '2318294704054686.000000000000000000',
        #     'market_cap': '43370130583253964.000000000000000000',
        #     'circulating_supply': '19588837.000000000000000000',
        #     'all_time_high': '3577014315.000000000000000000',
        #     'popularity_weight': 0,
        #     'freshness_weight': 0,
        # }
        marketType = 'spot'
        priceInfo = self.safe_value(ticker, 'order_book_info')
        marketId = self.safe_string(ticker, 'id')
        symbol = self.safe_symbol(marketId, market, None, marketType)
        high = self.safe_float(priceInfo, 'max', 0)
        low = self.safe_float(priceInfo, 'min', 0)
        last = self.safe_float(priceInfo, 'lastPrice', 0)
        change = self.safe_float(priceInfo, 'change', 0)
        quoteVolume = self.safe_float(priceInfo, '24h_quoteVolume', 0)
        return self.safe_ticker({
            'symbol': symbol,
            'timestamp': None,
            'datetime': None,
            'high': high,
            'low': low,
            'bid': None,
            'bidVolume': None,
            'ask': None,
            'askVolume': None,
            'vwap': None,
            'open': None,
            'close': last,
            'last': last,
            'previousClose': None,
            'change': change,
            'percentage': None,
            'average': None,
            'baseVolume': None,
            'quoteVolume': quoteVolume,
            'info': ticker,
        }, market)

    async def fetch_ohlcv(self, symbol: str, timeframe='1m', since: Int = None, limit: Int = None, params={}) -> List[list]:
        """
        fetches historical candlestick data containing the open, high, low, and close price, and the volume of a market
        :see: https://api-docs.bitpin.ir/#be8d9c51a2
        :param str symbol: unified symbol of the market to fetch OHLCV data for
        :param str timeframe: the length of time each candle represents
        :param int [since]: timestamp in ms of the earliest candle to fetch
        :param int [limit]: the maximum amount of candles to fetch
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns int[][]: A list of candles ordered, open, high, low, close, volume
        """
        await self.load_markets()
        market = self.market(symbol)
        endTime = Date.now()
        request = {
            'symbol': market['symbol'].replace('/', '_'),
            'from': (endTime / 1000) - (24 * 60 * 60),
            'to': endTime / 1000,
            'res': self.safe_string(self.timeframes, timeframe, timeframe),
        }
        if since is not None:
            request['from'] = since / 1000
        request['from'] = self.safe_integer(request, 'from')
        request['to'] = self.safe_integer(request, 'to')
        if timeframe is not None:
            request['res'] = self.safe_string(self.timeframes, timeframe, timeframe)
        response = await self.publicGetV1MktTvGetBars(request)
        ohlcvs = []
        for i in range(0, len(response)):
            ohlcvs.append([
                self.safe_value(response[i], 'ts'),
                self.safe_float(response[i], 'open'),
                self.safe_float(response[i], 'high'),
                self.safe_float(response[i], 'low'),
                self.safe_float(response[i], 'close'),
                self.safe_float(response[i], 'volume'),
            ])
        return self.parse_ohlcvs(ohlcvs, market, timeframe, since, limit)

    async def fetch_order_book(self, symbol: str, limit: Int = None, params={}) -> OrderBook:
        """
        fetches information on open orders with bid(buy) and ask(sell) prices, volumes and other data for multiple markets
        :see: https://api-docs.bitpin.ir/#be8d9c51a2
        :param str[]|None symbols: list of unified market symbols, all symbols fetched if None, default is None
        :param int [limit]: max number of entries per orderbook to return, default is None
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a dictionary of `order book structures <https://docs.ccxt.com/#/?id=order-book-structure>` indexed by market symbol
        """
        await self.load_markets()
        market = self.market(symbol)
        request = {
            'symbol': market['id'],
            'type': 'buy',
        }
        Buyresponse = await self.publicGetV2MthActives(request)
        request['type'] = 'sell'
        Sellresponse = await self.publicGetV2MthActives(request)
        BuyorderBook = self.safe_dict(Buyresponse, 'orders', {})
        SellorderBook = self.safe_dict(Sellresponse, 'orders', {})
        orderBook = {'bid': BuyorderBook, 'ask': SellorderBook}
        timestamp = Date.now()
        return self.parse_order_book(orderBook, symbol, timestamp, 'bid', 'ask', 'price', 'amount')

    def sign(self, path, api='public', method='GET', params={}, headers=None, body=None):
        query = self.omit(params, self.extract_params(path))
        url = self.urls['api'][api] + '/' + path
        if path == 'v1/mkt/tv/get_bars/':
            url = self.urls['api']['OHLCV'] + '/' + path + '?' + self.urlencode(query)
        if path == 'v2/mth/actives/':
            url = url + params['symbol'] + '/?type=' + params['type']
        headers = {'Content-Type': 'application/json'}
        return {'url': url, 'method': method, 'body': body, 'headers': headers}



# -*- coding: utf-8 -*-

# PLEASE DO NOT EDIT THIS FILE, IT IS GENERATED AND WILL BE OVERWRITTEN:
# https://github.com/ccxt/ccxt/blob/master/CONTRIBUTING.md#how-to-contribute-code

from ccxt.async_support.base.exchange import Exchange
from ccxt.abstract.independentreserve import ImplicitAPI
import hashlib
from ccxt.base.types import Balances, Currency, Int, Market, Num, Order, OrderBook, OrderSide, OrderType, Str, Ticker, Trade, TradingFees
from typing import List
from ccxt.base.decimal_to_precision import TICK_SIZE
from ccxt.base.precise import Precise


class independentreserve(Exchange, ImplicitAPI):

    def describe(self):
        return self.deep_extend(super(independentreserve, self).describe(), {
            'id': 'independentreserve',
            'name': 'Independent Reserve',
            'countries': ['AU', 'NZ'],  # Australia, New Zealand
            'rateLimit': 1000,
            'pro': True,
            'has': {
                'CORS': None,
                'spot': True,
                'margin': False,
                'swap': False,
                'future': False,
                'option': False,
                'addMargin': False,
                'cancelOrder': True,
                'closeAllPositions': False,
                'closePosition': False,
                'createOrder': True,
                'createReduceOnlyOrder': False,
                'createStopLimitOrder': False,
                'createStopMarketOrder': False,
                'createStopOrder': False,
                'fetchBalance': True,
                'fetchBorrowRateHistories': False,
                'fetchBorrowRateHistory': False,
                'fetchClosedOrders': True,
                'fetchCrossBorrowRate': False,
                'fetchCrossBorrowRates': False,
                'fetchDepositAddress': True,
                'fetchDepositAddresses': False,
                'fetchDepositAddressesByNetwork': False,
                'fetchFundingHistory': False,
                'fetchFundingRate': False,
                'fetchFundingRateHistory': False,
                'fetchFundingRates': False,
                'fetchIndexOHLCV': False,
                'fetchIsolatedBorrowRate': False,
                'fetchIsolatedBorrowRates': False,
                'fetchLeverage': False,
                'fetchLeverageTiers': False,
                'fetchMarginMode': False,
                'fetchMarkets': True,
                'fetchMarkOHLCV': False,
                'fetchMyTrades': True,
                'fetchOpenInterestHistory': False,
                'fetchOpenOrders': True,
                'fetchOrder': True,
                'fetchOrderBook': True,
                'fetchPosition': False,
                'fetchPositionHistory': False,
                'fetchPositionMode': False,
                'fetchPositions': False,
                'fetchPositionsForSymbol': False,
                'fetchPositionsHistory': False,
                'fetchPositionsRisk': False,
                'fetchPremiumIndexOHLCV': False,
                'fetchTicker': True,
                'fetchTrades': True,
                'fetchTradingFee': False,
                'fetchTradingFees': True,
                'reduceMargin': False,
                'setLeverage': False,
                'setMarginMode': False,
                'setPositionMode': False,
            },
            'urls': {
                'logo': 'https://user-images.githubusercontent.com/51840849/87182090-1e9e9080-c2ec-11ea-8e49-563db9a38f37.jpg',
                'api': {
                    'public': 'https://api.independentreserve.com/Public',
                    'private': 'https://api.independentreserve.com/Private',
                },
                'www': 'https://www.independentreserve.com',
                'doc': 'https://www.independentreserve.com/API',
            },
            'api': {
                'public': {
                    'get': [
                        'GetValidPrimaryCurrencyCodes',
                        'GetValidSecondaryCurrencyCodes',
                        'GetValidLimitOrderTypes',
                        'GetValidMarketOrderTypes',
                        'GetValidOrderTypes',
                        'GetValidTransactionTypes',
                        'GetMarketSummary',
                        'GetOrderBook',
                        'GetAllOrders',
                        'GetTradeHistorySummary',
                        'GetRecentTrades',
                        'GetFxRates',
                        'GetOrderMinimumVolumes',
                        'GetCryptoWithdrawalFees',
                    ],
                },
                'private': {
                    'post': [
                        'GetOpenOrders',
                        'GetClosedOrders',
                        'GetClosedFilledOrders',
                        'GetOrderDetails',
                        'GetAccounts',
                        'GetTransactions',
                        'GetFiatBankAccounts',
                        'GetDigitalCurrencyDepositAddress',
                        'GetDigitalCurrencyDepositAddresses',
                        'GetTrades',
                        'GetBrokerageFees',
                        'GetDigitalCurrencyWithdrawal',
                        'PlaceLimitOrder',
                        'PlaceMarketOrder',
                        'CancelOrder',
                        'SynchDigitalCurrencyDepositAddressWithBlockchain',
                        'RequestFiatWithdrawal',
                        'WithdrawFiatCurrency',
                        'WithdrawDigitalCurrency',
                    ],
                },
            },
            'fees': {
                'trading': {
                    'taker': self.parse_number('0.005'),
                    'maker': self.parse_number('0.005'),
                    'percentage': True,
                    'tierBased': False,
                },
            },
            'commonCurrencies': {
                'PLA': 'PlayChip',
            },
            'precisionMode': TICK_SIZE,
        })

    async def fetch_markets(self, params={}) -> List[Market]:
        """
        retrieves data on all markets for independentreserve
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict[]: an array of objects representing market data
        """
        baseCurrencies = await self.publicGetGetValidPrimaryCurrencyCodes(params)
        #     ['Xbt', 'Eth', 'Usdt', ...]
        quoteCurrencies = await self.publicGetGetValidSecondaryCurrencyCodes(params)
        #     ['Aud', 'Usd', 'Nzd', 'Sgd']
        limits = await self.publicGetGetOrderMinimumVolumes(params)
        #
        #     {
        #         "Xbt": 0.0001,
        #         "Eth": 0.001,
        #         "Ltc": 0.01,
        #         "Xrp": 1.0,
        #     }
        #
        result = []
        for i in range(0, len(baseCurrencies)):
            baseId = baseCurrencies[i]
            base = self.safe_currency_code(baseId)
            minAmount = self.safe_number(limits, baseId)
            for j in range(0, len(quoteCurrencies)):
                quoteId = quoteCurrencies[j]
                quote = self.safe_currency_code(quoteId)
                id = baseId + '/' + quoteId
                result.append({
                    'id': id,
                    'symbol': base + '/' + quote,
                    'base': base,
                    'quote': quote,
                    'settle': None,
                    'baseId': baseId,
                    'quoteId': quoteId,
                    'settleId': None,
                    'type': 'spot',
                    'spot': True,
                    'margin': False,
                    'swap': False,
                    'future': False,
                    'option': False,
                    'active': None,
                    'contract': False,
                    'linear': None,
                    'inverse': None,
                    'contractSize': None,
                    'expiry': None,
                    'expiryDatetime': None,
                    'strike': None,
                    'optionType': None,
                    'precision': {
                        'amount': None,
                        'price': None,
                    },
                    'limits': {
                        'leverage': {
                            'min': None,
                            'max': None,
                        },
                        'amount': {
                            'min': minAmount,
                            'max': None,
                        },
                        'price': {
                            'min': None,
                            'max': None,
                        },
                        'cost': {
                            'min': None,
                            'max': None,
                        },
                    },
                    'created': None,
                    'info': id,
                })
        return result

    def parse_balance(self, response) -> Balances:
        result: dict = {'info': response}
        for i in range(0, len(response)):
            balance = response[i]
            currencyId = self.safe_string(balance, 'CurrencyCode')
            code = self.safe_currency_code(currencyId)
            account = self.account()
            account['free'] = self.safe_string(balance, 'AvailableBalance')
            account['total'] = self.safe_string(balance, 'TotalBalance')
            result[code] = account
        return self.safe_balance(result)

    async def fetch_balance(self, params={}) -> Balances:
        """
        query for balance and get the amount of funds available for trading or funds locked in orders
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a `balance structure <https://docs.ccxt.com/#/?id=balance-structure>`
        """
        await self.load_markets()
        response = await self.privatePostGetAccounts(params)
        return self.parse_balance(response)

    async def fetch_order_book(self, symbol: str, limit: Int = None, params={}) -> OrderBook:
        """
        fetches information on open orders with bid(buy) and ask(sell) prices, volumes and other data
        :param str symbol: unified symbol of the market to fetch the order book for
        :param int [limit]: the maximum amount of order book entries to return
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: A dictionary of `order book structures <https://docs.ccxt.com/#/?id=order-book-structure>` indexed by market symbols
        """
        await self.load_markets()
        market = self.market(symbol)
        request: dict = {
            'primaryCurrencyCode': market['baseId'],
            'secondaryCurrencyCode': market['quoteId'],
        }
        response = await self.publicGetGetOrderBook(self.extend(request, params))
        timestamp = self.parse8601(self.safe_string(response, 'CreatedTimestampUtc'))
        return self.parse_order_book(response, market['symbol'], timestamp, 'BuyOrders', 'SellOrders', 'Price', 'Volume')

    def parse_ticker(self, ticker: dict, market: Market = None) -> Ticker:
        # {
        #     "DayHighestPrice":43489.49,
        #     "DayLowestPrice":41998.32,
        #     "DayAvgPrice":42743.9,
        #     "DayVolumeXbt":44.54515625000,
        #     "DayVolumeXbtInSecondaryCurrrency":0.12209818,
        #     "CurrentLowestOfferPrice":43619.64,
        #     "CurrentHighestBidPrice":43153.58,
        #     "LastPrice":43378.43,
        #     "PrimaryCurrencyCode":"Xbt",
        #     "SecondaryCurrencyCode":"Usd",
        #     "CreatedTimestampUtc":"2022-01-14T22:52:29.5029223Z"
        # }
        timestamp = self.parse8601(self.safe_string(ticker, 'CreatedTimestampUtc'))
        baseId = self.safe_string(ticker, 'PrimaryCurrencyCode')
        quoteId = self.safe_string(ticker, 'SecondaryCurrencyCode')
        defaultMarketId = None
        if (baseId is not None) and (quoteId is not None):
            defaultMarketId = baseId + '/' + quoteId
        market = self.safe_market(defaultMarketId, market, '/')
        symbol = market['symbol']
        last = self.safe_string(ticker, 'LastPrice')
        return self.safe_ticker({
            'symbol': symbol,
            'timestamp': timestamp,
            'datetime': self.iso8601(timestamp),
            'high': self.safe_string(ticker, 'DayHighestPrice'),
            'low': self.safe_string(ticker, 'DayLowestPrice'),
            'bid': self.safe_string(ticker, 'CurrentHighestBidPrice'),
            'bidVolume': None,
            'ask': self.safe_string(ticker, 'CurrentLowestOfferPrice'),
            'askVolume': None,
            'vwap': None,
            'open': None,
            'close': last,
            'last': last,
            'previousClose': None,
            'change': None,
            'percentage': None,
            'average': self.safe_string(ticker, 'DayAvgPrice'),
            'baseVolume': self.safe_string(ticker, 'DayVolumeXbtInSecondaryCurrrency'),
            'quoteVolume': None,
            'info': ticker,
        }, market)

    async def fetch_ticker(self, symbol: str, params={}) -> Ticker:
        """
        fetches a price ticker, a statistical calculation with the information calculated over the past 24 hours for a specific market
        :param str symbol: unified symbol of the market to fetch the ticker for
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a `ticker structure <https://docs.ccxt.com/#/?id=ticker-structure>`
        """
        await self.load_markets()
        market = self.market(symbol)
        request: dict = {
            'primaryCurrencyCode': market['baseId'],
            'secondaryCurrencyCode': market['quoteId'],
        }
        response = await self.publicGetGetMarketSummary(self.extend(request, params))
        # {
        #     "DayHighestPrice":43489.49,
        #     "DayLowestPrice":41998.32,
        #     "DayAvgPrice":42743.9,
        #     "DayVolumeXbt":44.54515625000,
        #     "DayVolumeXbtInSecondaryCurrrency":0.12209818,
        #     "CurrentLowestOfferPrice":43619.64,
        #     "CurrentHighestBidPrice":43153.58,
        #     "LastPrice":43378.43,
        #     "PrimaryCurrencyCode":"Xbt",
        #     "SecondaryCurrencyCode":"Usd",
        #     "CreatedTimestampUtc":"2022-01-14T22:52:29.5029223Z"
        # }
        return self.parse_ticker(response, market)

    def parse_order(self, order: dict, market: Market = None) -> Order:
        #
        # fetchOrder
        #
        #     {
        #         "OrderGuid": "c7347e4c-b865-4c94-8f74-d934d4b0b177",
        #         "CreatedTimestampUtc": "2014-09-23T12:39:34.3817763Z",
        #         "Type": "MarketBid",
        #         "VolumeOrdered": 5.0,
        #         "VolumeFilled": 5.0,
        #         "Price": null,
        #         "AvgPrice": 100.0,
        #         "ReservedAmount": 0.0,
        #         "Status": "Filled",
        #         "PrimaryCurrencyCode": "Xbt",
        #         "SecondaryCurrencyCode": "Usd"
        #     }
        #
        # fetchOpenOrders & fetchClosedOrders
        #
        #     {
        #         "OrderGuid": "b8f7ad89-e4e4-4dfe-9ea3-514d38b5edb3",
        #         "CreatedTimestampUtc": "2020-09-08T03:04:18.616367Z",
        #         "OrderType": "LimitOffer",
        #         "Volume": 0.0005,
        #         "Outstanding": 0.0005,
        #         "Price": 113885.83,
        #         "AvgPrice": 113885.83,
        #         "Value": 56.94,
        #         "Status": "Open",
        #         "PrimaryCurrencyCode": "Xbt",
        #         "SecondaryCurrencyCode": "Usd",
        #         "FeePercent": 0.005,
        #     }
        #
        # cancelOrder
        #
        #    {
        #        "AvgPrice": 455.48,
        #        "CreatedTimestampUtc": "2022-08-05T06:42:11.3032208Z",
        #        "OrderGuid": "719c495c-a39e-4884-93ac-280b37245037",
        #        "Price": 485.76,
        #        "PrimaryCurrencyCode": "Xbt",
        #        "ReservedAmount": 0.358,
        #        "SecondaryCurrencyCode": "Usd",
        #        "Status": "Cancelled",
        #        "Type": "LimitOffer",
        #        "VolumeFilled": 0,
        #        "VolumeOrdered": 0.358
        #    }
        symbol = None
        baseId = self.safe_string(order, 'PrimaryCurrencyCode')
        quoteId = self.safe_string(order, 'SecondaryCurrencyCode')
        base = None
        quote = None
        if (baseId is not None) and (quoteId is not None):
            base = self.safe_currency_code(baseId)
            quote = self.safe_currency_code(quoteId)
            symbol = base + '/' + quote
        elif market is not None:
            symbol = market['symbol']
            base = market['base']
            quote = market['quote']
        orderType = self.safe_string_2(order, 'Type', 'OrderType')
        side = None
        if orderType is not None:
            if orderType.find('Bid') >= 0:
                side = 'buy'
            elif orderType.find('Offer') >= 0:
                side = 'sell'
            if orderType.find('Market') >= 0:
                orderType = 'market'
            elif orderType.find('Limit') >= 0:
                orderType = 'limit'
        timestamp = self.parse8601(self.safe_string(order, 'CreatedTimestampUtc'))
        filled = self.safe_string(order, 'VolumeFilled')
        feeRate = self.safe_string(order, 'FeePercent')
        feeCost = None
        if feeRate is not None and filled is not None:
            feeCost = Precise.string_mul(feeRate, filled)
        return self.safe_order({
            'info': order,
            'id': self.safe_string(order, 'OrderGuid'),
            'clientOrderId': None,
            'timestamp': timestamp,
            'datetime': self.iso8601(timestamp),
            'lastTradeTimestamp': None,
            'symbol': symbol,
            'type': orderType,
            'timeInForce': None,
            'postOnly': None,
            'side': side,
            'price': self.safe_string(order, 'Price'),
            'stopPrice': None,
            'triggerPrice': None,
            'cost': self.safe_string(order, 'Value'),
            'average': self.safe_string(order, 'AvgPrice'),
            'amount': self.safe_string_2(order, 'VolumeOrdered', 'Volume'),
            'filled': filled,
            'remaining': self.safe_string(order, 'Outstanding'),
            'status': self.parse_order_status(self.safe_string(order, 'Status')),
            'fee': {
                'rate': feeRate,
                'cost': feeCost,
                'currency': base,
            },
            'trades': None,
        }, market)

    def parse_order_status(self, status: Str):
        statuses: dict = {
            'Open': 'open',
            'PartiallyFilled': 'open',
            'Filled': 'closed',
            'PartiallyFilledAndCancelled': 'canceled',
            'Cancelled': 'canceled',
            'PartiallyFilledAndExpired': 'canceled',
            'Expired': 'canceled',
        }
        return self.safe_string(statuses, status, status)

    async def fetch_order(self, id: str, symbol: Str = None, params={}):
        """
        fetches information on an order made by the user
        :param str symbol: unified symbol of the market the order was made in
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: An `order structure <https://docs.ccxt.com/#/?id=order-structure>`
        """
        await self.load_markets()
        response = await self.privatePostGetOrderDetails(self.extend({
            'orderGuid': id,
        }, params))
        market = None
        if symbol is not None:
            market = self.market(symbol)
        return self.parse_order(response, market)

    async def fetch_open_orders(self, symbol: Str = None, since: Int = None, limit: Int = None, params={}) -> List[Order]:
        """
        fetch all unfilled currently open orders
        :param str symbol: unified market symbol
        :param int [since]: the earliest time in ms to fetch open orders for
        :param int [limit]: the maximum number of  open orders structures to retrieve
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns Order[]: a list of `order structures <https://docs.ccxt.com/#/?id=order-structure>`
        """
        await self.load_markets()
        request = self.ordered({})
        market = None
        if symbol is not None:
            market = self.market(symbol)
            request['primaryCurrencyCode'] = market['baseId']
            request['secondaryCurrencyCode'] = market['quoteId']
        if limit is None:
            limit = 50
        request['pageIndex'] = 1
        request['pageSize'] = limit
        response = await self.privatePostGetOpenOrders(self.extend(request, params))
        data = self.safe_list(response, 'Data', [])
        return self.parse_orders(data, market, since, limit)

    async def fetch_closed_orders(self, symbol: Str = None, since: Int = None, limit: Int = None, params={}) -> List[Order]:
        """
        fetches information on multiple closed orders made by the user
        :param str symbol: unified market symbol of the market orders were made in
        :param int [since]: the earliest time in ms to fetch orders for
        :param int [limit]: the maximum number of order structures to retrieve
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns Order[]: a list of `order structures <https://docs.ccxt.com/#/?id=order-structure>`
        """
        await self.load_markets()
        request = self.ordered({})
        market = None
        if symbol is not None:
            market = self.market(symbol)
            request['primaryCurrencyCode'] = market['baseId']
            request['secondaryCurrencyCode'] = market['quoteId']
        if limit is None:
            limit = 50
        request['pageIndex'] = 1
        request['pageSize'] = limit
        response = await self.privatePostGetClosedOrders(self.extend(request, params))
        data = self.safe_list(response, 'Data', [])
        return self.parse_orders(data, market, since, limit)

    async def fetch_my_trades(self, symbol: Str = None, since: Int = None, limit: Int = 50, params={}):
        """
        fetch all trades made by the user
        :param str symbol: unified market symbol
        :param int [since]: the earliest time in ms to fetch trades for
        :param int [limit]: the maximum number of trades structures to retrieve
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns Trade[]: a list of `trade structures <https://docs.ccxt.com/#/?id=trade-structure>`
        """
        await self.load_markets()
        pageIndex = self.safe_integer(params, 'pageIndex', 1)
        if limit is None:
            limit = 50
        request = self.ordered({
            'pageIndex': pageIndex,
            'pageSize': limit,
        })
        response = await self.privatePostGetTrades(self.extend(request, params))
        market = None
        if symbol is not None:
            market = self.market(symbol)
        return self.parse_trades(response['Data'], market, since, limit)

    def parse_trade(self, trade: dict, market: Market = None) -> Trade:
        timestamp = self.parse8601(trade['TradeTimestampUtc'])
        id = self.safe_string(trade, 'TradeGuid')
        orderId = self.safe_string(trade, 'OrderGuid')
        priceString = self.safe_string_2(trade, 'Price', 'SecondaryCurrencyTradePrice')
        amountString = self.safe_string_2(trade, 'VolumeTraded', 'PrimaryCurrencyAmount')
        price = self.parse_number(priceString)
        amount = self.parse_number(amountString)
        cost = self.parse_number(Precise.string_mul(priceString, amountString))
        baseId = self.safe_string(trade, 'PrimaryCurrencyCode')
        quoteId = self.safe_string(trade, 'SecondaryCurrencyCode')
        marketId = None
        if (baseId is not None) and (quoteId is not None):
            marketId = baseId + '/' + quoteId
        symbol = self.safe_symbol(marketId, market, '/')
        side = self.safe_string(trade, 'OrderType')
        if side is not None:
            if side.find('Bid') >= 0:
                side = 'buy'
            elif side.find('Offer') >= 0:
                side = 'sell'
        return self.safe_trade({
            'id': id,
            'info': trade,
            'timestamp': timestamp,
            'datetime': self.iso8601(timestamp),
            'symbol': symbol,
            'order': orderId,
            'type': None,
            'side': side,
            'takerOrMaker': None,
            'price': price,
            'amount': amount,
            'cost': cost,
            'fee': None,
        }, market)

    async def fetch_trades(self, symbol: str, since: Int = None, limit: Int = None, params={}) -> List[Trade]:
        """
        get the list of most recent trades for a particular symbol
        :param str symbol: unified symbol of the market to fetch trades for
        :param int [since]: timestamp in ms of the earliest trade to fetch
        :param int [limit]: the maximum amount of trades to fetch
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns Trade[]: a list of `trade structures <https://docs.ccxt.com/#/?id=public-trades>`
        """
        await self.load_markets()
        market = self.market(symbol)
        request: dict = {
            'primaryCurrencyCode': market['baseId'],
            'secondaryCurrencyCode': market['quoteId'],
            'numberOfRecentTradesToRetrieve': 50,  # max = 50
        }
        response = await self.publicGetGetRecentTrades(self.extend(request, params))
        return self.parse_trades(response['Trades'], market, since, limit)

    async def fetch_trading_fees(self, params={}) -> TradingFees:
        """
        fetch the trading fees for multiple markets
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: a dictionary of `fee structures <https://docs.ccxt.com/#/?id=fee-structure>` indexed by market symbols
        """
        await self.load_markets()
        response = await self.privatePostGetBrokerageFees(params)
        #
        #     [
        #         {
        #             "CurrencyCode": "Xbt",
        #             "Fee": 0.005
        #         }
        #         ...
        #     ]
        #
        fees: dict = {}
        for i in range(0, len(response)):
            fee = response[i]
            currencyId = self.safe_string(fee, 'CurrencyCode')
            code = self.safe_currency_code(currencyId)
            tradingFee = self.safe_number(fee, 'Fee')
            fees[code] = {
                'info': fee,
                'fee': tradingFee,
            }
        result: dict = {}
        for i in range(0, len(self.symbols)):
            symbol = self.symbols[i]
            market = self.market(symbol)
            fee = self.safe_value(fees, market['base'], {})
            result[symbol] = {
                'info': self.safe_value(fee, 'info'),
                'symbol': symbol,
                'maker': self.safe_number(fee, 'fee'),
                'taker': self.safe_number(fee, 'fee'),
                'percentage': True,
                'tierBased': True,
            }
        return result

    async def create_order(self, symbol: str, type: OrderType, side: OrderSide, amount: float, price: Num = None, params={}):
        """
        create a trade order
        :param str symbol: unified symbol of the market to create an order in
        :param str type: 'market' or 'limit'
        :param str side: 'buy' or 'sell'
        :param float amount: how much of currency you want to trade in units of base currency
        :param float [price]: the price at which the order is to be fullfilled, in units of the quote currency, ignored in market orders
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: an `order structure <https://docs.ccxt.com/#/?id=order-structure>`
        """
        await self.load_markets()
        market = self.market(symbol)
        orderType = self.capitalize(type)
        orderType += 'Offer' if (side == 'sell') else 'Bid'
        request = self.ordered({
            'primaryCurrencyCode': market['baseId'],
            'secondaryCurrencyCode': market['quoteId'],
            'orderType': orderType,
        })
        response = None
        request['volume'] = amount
        if type == 'limit':
            request['price'] = price
            response = await self.privatePostPlaceLimitOrder(self.extend(request, params))
        else:
            response = await self.privatePostPlaceMarketOrder(self.extend(request, params))
        return self.safe_order({
            'info': response,
            'id': response['OrderGuid'],
        }, market)

    async def cancel_order(self, id: str, symbol: Str = None, params={}):
        """
        cancels an open order
        :see: https://www.independentreserve.com/features/api#CancelOrder
        :param str id: order id
        :param str symbol: unified symbol of the market the order was made in
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: An `order structure <https://docs.ccxt.com/#/?id=order-structure>`
        """
        await self.load_markets()
        request: dict = {
            'orderGuid': id,
        }
        response = await self.privatePostCancelOrder(self.extend(request, params))
        #
        #    {
        #        "AvgPrice": 455.48,
        #        "CreatedTimestampUtc": "2022-08-05T06:42:11.3032208Z",
        #        "OrderGuid": "719c495c-a39e-4884-93ac-280b37245037",
        #        "Price": 485.76,
        #        "PrimaryCurrencyCode": "Xbt",
        #        "ReservedAmount": 0.358,
        #        "SecondaryCurrencyCode": "Usd",
        #        "Status": "Cancelled",
        #        "Type": "LimitOffer",
        #        "VolumeFilled": 0,
        #        "VolumeOrdered": 0.358
        #    }
        #
        return self.parse_order(response)

    async def fetch_deposit_address(self, code: str, params={}):
        """
        fetch the deposit address for a currency associated with self account
        :see: https://www.independentreserve.com/features/api#GetDigitalCurrencyDepositAddress
        :param str code: unified currency code
        :param dict [params]: extra parameters specific to the exchange API endpoint
        :returns dict: an `address structure <https://docs.ccxt.com/#/?id=address-structure>`
        """
        await self.load_markets()
        currency = self.currency(code)
        request: dict = {
            'primaryCurrencyCode': currency['id'],
        }
        response = await self.privatePostGetDigitalCurrencyDepositAddress(self.extend(request, params))
        #
        #    {
        #        Tag: '3307446684',
        #        DepositAddress: 'GCCQH4HACMRAD56EZZZ4TOIDQQRVNADMJ35QOFWF4B2VQGODMA2WVQ22',
        #        LastCheckedTimestampUtc: '2024-02-20T11:13:35.6912985Z',
        #        NextUpdateTimestampUtc: '2024-02-20T11:14:56.5112394Z'
        #    }
        #
        return self.parse_deposit_address(response)

    def parse_deposit_address(self, depositAddress, currency: Currency = None):
        #
        #    {
        #        Tag: '3307446684',
        #        DepositAddress: 'GCCQH4HACMRAD56EZZZ4TOIDQQRVNADMJ35QOFWF4B2VQGODMA2WVQ22',
        #        LastCheckedTimestampUtc: '2024-02-20T11:13:35.6912985Z',
        #        NextUpdateTimestampUtc: '2024-02-20T11:14:56.5112394Z'
        #    }
        #
        address = self.safe_string(depositAddress, 'DepositAddress')
        self.check_address(address)
        return {
            'info': depositAddress,
            'currency': self.safe_string(currency, 'code'),
            'address': address,
            'tag': self.safe_string(depositAddress, 'Tag'),
            'network': None,
        }

    def sign(self, path, api='public', method='GET', params={}, headers=None, body=None):
        url = self.urls['api'][api] + '/' + path
        if api == 'public':
            if params:
                url += '?' + self.urlencode(params)
        else:
            self.check_required_credentials()
            nonce = self.nonce()
            auth = [
                url,
                'apiKey=' + self.apiKey,
                'nonce=' + str(nonce),
            ]
            keys = list(params.keys())
            for i in range(0, len(keys)):
                key = keys[i]
                value = str(params[key])
                auth.append(key + '=' + value)
            message = ','.join(auth)
            signature = self.hmac(self.encode(message), self.encode(self.secret), hashlib.sha256)
            query = self.ordered({})
            query['apiKey'] = self.apiKey
            query['nonce'] = nonce
            query['signature'] = signature.upper()
            for i in range(0, len(keys)):
                key = keys[i]
                query[key] = params[key]
            body = self.json(query)
            headers = {'Content-Type': 'application/json'}
        return {'url': url, 'method': method, 'body': body, 'headers': headers}



from ccxt.base.types import Entry


class ImplicitAPI:
    trader_private_get_v2_account = traderPrivateGetV2Account = Entry('v2/account', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_orders = traderPrivateGetV2Orders = Entry('v2/orders', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_orders_order_id = traderPrivateGetV2OrdersOrderId = Entry('v2/orders/{order_id}', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_positions = traderPrivateGetV2Positions = Entry('v2/positions', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_positions_symbol_or_asset_id = traderPrivateGetV2PositionsSymbolOrAssetId = Entry('v2/positions/{symbol_or_asset_id}', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_account_portfolio_history = traderPrivateGetV2AccountPortfolioHistory = Entry('v2/account/portfolio/history', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_watchlists = traderPrivateGetV2Watchlists = Entry('v2/watchlists', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_watchlists_watchlist_id = traderPrivateGetV2WatchlistsWatchlistId = Entry('v2/watchlists/{watchlist_id}', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_watchlists_by_name = traderPrivateGetV2WatchlistsByName = Entry('v2/watchlists:by_name', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_account_configurations = traderPrivateGetV2AccountConfigurations = Entry('v2/account/configurations', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_account_activities = traderPrivateGetV2AccountActivities = Entry('v2/account/activities', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_account_activities_activity_type = traderPrivateGetV2AccountActivitiesActivityType = Entry('v2/account/activities/{activity_type}', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_calendar = traderPrivateGetV2Calendar = Entry('v2/calendar', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_clock = traderPrivateGetV2Clock = Entry('v2/clock', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_assets = traderPrivateGetV2Assets = Entry('v2/assets', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_assets_symbol_or_asset_id = traderPrivateGetV2AssetsSymbolOrAssetId = Entry('v2/assets/{symbol_or_asset_id}', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_corporate_actions_announcements_id = traderPrivateGetV2CorporateActionsAnnouncementsId = Entry('v2/corporate_actions/announcements/{id}', ['trader', 'private'], 'GET', {})
    trader_private_get_v2_corporate_actions_announcements = traderPrivateGetV2CorporateActionsAnnouncements = Entry('v2/corporate_actions/announcements', ['trader', 'private'], 'GET', {})
    trader_private_post_v2_orders = traderPrivatePostV2Orders = Entry('v2/orders', ['trader', 'private'], 'POST', {})
    trader_private_post_v2_watchlists = traderPrivatePostV2Watchlists = Entry('v2/watchlists', ['trader', 'private'], 'POST', {})
    trader_private_post_v2_watchlists_watchlist_id = traderPrivatePostV2WatchlistsWatchlistId = Entry('v2/watchlists/{watchlist_id}', ['trader', 'private'], 'POST', {})
    trader_private_post_v2_watchlists_by_name = traderPrivatePostV2WatchlistsByName = Entry('v2/watchlists:by_name', ['trader', 'private'], 'POST', {})
    trader_private_put_v2_watchlists_watchlist_id = traderPrivatePutV2WatchlistsWatchlistId = Entry('v2/watchlists/{watchlist_id}', ['trader', 'private'], 'PUT', {})
    trader_private_put_v2_watchlists_by_name = traderPrivatePutV2WatchlistsByName = Entry('v2/watchlists:by_name', ['trader', 'private'], 'PUT', {})
    trader_private_patch_v2_orders_order_id = traderPrivatePatchV2OrdersOrderId = Entry('v2/orders/{order_id}', ['trader', 'private'], 'PATCH', {})
    trader_private_patch_v2_account_configurations = traderPrivatePatchV2AccountConfigurations = Entry('v2/account/configurations', ['trader', 'private'], 'PATCH', {})
    trader_private_delete_v2_orders = traderPrivateDeleteV2Orders = Entry('v2/orders', ['trader', 'private'], 'DELETE', {})
    trader_private_delete_v2_orders_order_id = traderPrivateDeleteV2OrdersOrderId = Entry('v2/orders/{order_id}', ['trader', 'private'], 'DELETE', {})
    trader_private_delete_v2_positions = traderPrivateDeleteV2Positions = Entry('v2/positions', ['trader', 'private'], 'DELETE', {})
    trader_private_delete_v2_positions_symbol_or_asset_id = traderPrivateDeleteV2PositionsSymbolOrAssetId = Entry('v2/positions/{symbol_or_asset_id}', ['trader', 'private'], 'DELETE', {})
    trader_private_delete_v2_watchlists_watchlist_id = traderPrivateDeleteV2WatchlistsWatchlistId = Entry('v2/watchlists/{watchlist_id}', ['trader', 'private'], 'DELETE', {})
    trader_private_delete_v2_watchlists_by_name = traderPrivateDeleteV2WatchlistsByName = Entry('v2/watchlists:by_name', ['trader', 'private'], 'DELETE', {})
    trader_private_delete_v2_watchlists_watchlist_id_symbol = traderPrivateDeleteV2WatchlistsWatchlistIdSymbol = Entry('v2/watchlists/{watchlist_id}/{symbol}', ['trader', 'private'], 'DELETE', {})
    market_public_get_v1beta3_crypto_loc_bars = marketPublicGetV1beta3CryptoLocBars = Entry('v1beta3/crypto/{loc}/bars', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_latest_bars = marketPublicGetV1beta3CryptoLocLatestBars = Entry('v1beta3/crypto/{loc}/latest/bars', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_latest_orderbooks = marketPublicGetV1beta3CryptoLocLatestOrderbooks = Entry('v1beta3/crypto/{loc}/latest/orderbooks', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_latest_quotes = marketPublicGetV1beta3CryptoLocLatestQuotes = Entry('v1beta3/crypto/{loc}/latest/quotes', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_latest_trades = marketPublicGetV1beta3CryptoLocLatestTrades = Entry('v1beta3/crypto/{loc}/latest/trades', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_quotes = marketPublicGetV1beta3CryptoLocQuotes = Entry('v1beta3/crypto/{loc}/quotes', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_snapshots = marketPublicGetV1beta3CryptoLocSnapshots = Entry('v1beta3/crypto/{loc}/snapshots', ['market', 'public'], 'GET', {})
    market_public_get_v1beta3_crypto_loc_trades = marketPublicGetV1beta3CryptoLocTrades = Entry('v1beta3/crypto/{loc}/trades', ['market', 'public'], 'GET', {})
    market_private_get_v1beta1_corporate_actions = marketPrivateGetV1beta1CorporateActions = Entry('v1beta1/corporate-actions', ['market', 'private'], 'GET', {})
    market_private_get_v1beta1_forex_latest_rates = marketPrivateGetV1beta1ForexLatestRates = Entry('v1beta1/forex/latest/rates', ['market', 'private'], 'GET', {})
    market_private_get_v1beta1_forex_rates = marketPrivateGetV1beta1ForexRates = Entry('v1beta1/forex/rates', ['market', 'private'], 'GET', {})
    market_private_get_v1beta1_logos_symbol = marketPrivateGetV1beta1LogosSymbol = Entry('v1beta1/logos/{symbol}', ['market', 'private'], 'GET', {})
    market_private_get_v1beta1_news = marketPrivateGetV1beta1News = Entry('v1beta1/news', ['market', 'private'], 'GET', {})
    market_private_get_v1beta1_screener_stocks_most_actives = marketPrivateGetV1beta1ScreenerStocksMostActives = Entry('v1beta1/screener/stocks/most-actives', ['market', 'private'], 'GET', {})
    market_private_get_v1beta1_screener_market_type_movers = marketPrivateGetV1beta1ScreenerMarketTypeMovers = Entry('v1beta1/screener/{market_type}/movers', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_auctions = marketPrivateGetV2StocksAuctions = Entry('v2/stocks/auctions', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_bars = marketPrivateGetV2StocksBars = Entry('v2/stocks/bars', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_bars_latest = marketPrivateGetV2StocksBarsLatest = Entry('v2/stocks/bars/latest', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_meta_conditions_ticktype = marketPrivateGetV2StocksMetaConditionsTicktype = Entry('v2/stocks/meta/conditions/{ticktype}', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_meta_exchanges = marketPrivateGetV2StocksMetaExchanges = Entry('v2/stocks/meta/exchanges', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_quotes = marketPrivateGetV2StocksQuotes = Entry('v2/stocks/quotes', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_quotes_latest = marketPrivateGetV2StocksQuotesLatest = Entry('v2/stocks/quotes/latest', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_snapshots = marketPrivateGetV2StocksSnapshots = Entry('v2/stocks/snapshots', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_trades = marketPrivateGetV2StocksTrades = Entry('v2/stocks/trades', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_trades_latest = marketPrivateGetV2StocksTradesLatest = Entry('v2/stocks/trades/latest', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_auctions = marketPrivateGetV2StocksSymbolAuctions = Entry('v2/stocks/{symbol}/auctions', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_bars = marketPrivateGetV2StocksSymbolBars = Entry('v2/stocks/{symbol}/bars', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_bars_latest = marketPrivateGetV2StocksSymbolBarsLatest = Entry('v2/stocks/{symbol}/bars/latest', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_quotes = marketPrivateGetV2StocksSymbolQuotes = Entry('v2/stocks/{symbol}/quotes', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_quotes_latest = marketPrivateGetV2StocksSymbolQuotesLatest = Entry('v2/stocks/{symbol}/quotes/latest', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_snapshot = marketPrivateGetV2StocksSymbolSnapshot = Entry('v2/stocks/{symbol}/snapshot', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_trades = marketPrivateGetV2StocksSymbolTrades = Entry('v2/stocks/{symbol}/trades', ['market', 'private'], 'GET', {})
    market_private_get_v2_stocks_symbol_trades_latest = marketPrivateGetV2StocksSymbolTradesLatest = Entry('v2/stocks/{symbol}/trades/latest', ['market', 'private'], 'GET', {})



import os
import sys

root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
sys.path.append(root)

# ----------------------------------------------------------------------------

# PLEASE DO NOT EDIT THIS FILE, IT IS GENERATED AND WILL BE OVERWRITTEN:
# https://github.com/ccxt/ccxt/blob/master/CONTRIBUTING.md#how-to-contribute-code

# ----------------------------------------------------------------------------
# -*- coding: utf-8 -*-



def test_status(exchange, skipped_properties, method, entry, now):
    format = {
        'info': {},
        'status': 'ok',
        'updated': 1650000000000,
        'eta': 1660000000000,
        'url': 'https://example.com',
    }
